#ifndef QLX_H
#define QLX_H

/* 0, 111, 100 are qmail-local success, soft, hard */

#define QLX_USAGE 112
#define QLX_BUG 101
#define QLX_ROOT 113
#define QLX_NFS 115
#define QLX_NOALIAS 116
#define QLX_CDB 117
#define QLX_SYS 118
#define QLX_NOMEM 119
#define QLX_EXECSOFT 120
#define QLX_EXECPW 121
#define QLX_EXECHARD 126

#endif
